/*  $Id: NodeAddress.h,v 1.29 2003/05/22 16:43:08 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @file

   This file contains the declarations for the NodeAddress class.

   @author Holger Wunsch */

#ifndef __NODEADDRESS_H
#define __NODEADDRESS_H

#include <node.h>
#include <qarray.h>

/** The NodeAddress class provides a means to uniquely identify nodes on screen.

    Although feature structures can contain shared data, the visible
    properties of two instances of a shared feature structure (e.g.
    visible vs. hidden, expanded vs. collapsed) can be different. Therefore,
    every node that can be displayed must have a unique address
    that not only encodes which data node it is associated with but also its
    exact position in the overall feature structure.
    This unique address is essentially a path from the topmost node in the
    overall structure to the substructure it identifies, very much like paths
    in standard feature structure descriptions.
    The format of a node address is as follows:
    Every single node in the data structure passed to grisu specifies an
    integer number in the "id" field of the record (see the communications
    protocol EBNF). This number is converted into an array of individual
    integers, each representing one indivdual digit, so 2436 becomes
    [2,4,3,6]. The full address of a node is calculated by concatenating
    all the id's of all the nodes from the root to this node. Node id's are
    prepended; thus the id of the root node will always end up to be the rightmost.
    The two uppper bits in each entry are reserved, where 0x80 indicates
    that the id of a new node starts at this position, and 0x40 indicates
    that the id of a node ends at this position. 
    
    @author Holger Wunsch */
class NodeAddress {
 public:
  /** An array that contains the node address. */
  unsigned char* addr;

  
  /** The length of the addr array. */
  unsigned int len;


  /** Default constructor; creates a new, empty NodeAddress. Note that an
      empty node address is not equal to the node address of the root node,
      which contains exactly one node ID, the node ID of the root node in the
      data package. */
  NodeAddress();


  /** Creates a new node address for the node <code>nd</code>, where the
      node address of <code>nd</code>s parent node is <code>addrParent</code>.
      @param addrParent the node address of the parent node
      @param nd the node for which the new address is to be created. */
  NodeAddress(const NodeAddress& addrParent, GenericNode* nd);


  /** Creates a new node address leading to a node with node ID <code>id</code>.
      @param addrParent the node address of the parent node
      @param id the node ID of the node which the new address will be created for. */
  NodeAddress(const NodeAddress& addrParent, char* id);


  /** Copy constructor. */
  NodeAddress(NodeAddress&);


  /** Deletes all data members and destroys the instance. */
  ~NodeAddress();


  /** Tests whether a node address contains a given node ID.
      @param id the node ID to be tested
      @return <code>true</code>, if the ID occurs in the node address, <code>false</code> otherwise. */
  bool contains(char* id) const;

  
  /** Equality operator.
      @param na the node address this node address is to be compared to.
      @return <code>true</code> if the two node addresses are equal, <code>false</code> otherwise. */
  bool operator == (const NodeAddress& na) const;


  /** Assignment operator.
      @param na the node address that is to be assigned to this node address.
      @return A NodeAddress object containing the new value. */
  NodeAddress operator = (const NodeAddress& na);


  /** Calculates the node address of the immediate parent node.
      @param pa <code>[out]</code> the parent address will be assigned to the object passed in <code>pa</code>.
      <code>pa</code> must be a valid instance of a NodeAddress (preferably created with the default constructor). */
  void parentAddress(NodeAddress& pa);

  
  /** Returns the node ID of the node this address points to in a static buffer.
      @return a pointer to a static <code>char</code> buffer that contains the node ID. */
  const char* nodeID(void) const;


  /** Prints the node address to <code>stdout</code>. */
  void print() const;


  /** Prints the node address to a string buffer. */
  void sprint(char* buf);
};

#endif
