/*  $Id: canvas.h,v 1.32 2003/05/22 16:43:09 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file

    Definition of the Canvas class.

    @author Holger Wunsch */

#ifndef __CANVAS_H_
#define __CANVAS_H_

#include <qscrbar.h>
#include <qwidget.h>
#include <qkeycode.h>
#include <qprinter.h>
#include <kapp.h>
#include <kiconloader.h>
#include <kstatusbar.h>
#include <ktoolbar.h>
#include <kmenubar.h>
#include <keditcl.h>
#include <qpoint.h>

#include <kpopupmenu.h>

#include "data.h"
#include "options.h"

#include "slider.h"

#include "node.h"
#include "NodeAddress.h"
#include "dispnode.h"
#include "idtree.h"

#include "find.h"


/** Indicates that the node to be drawn is inside of a list */
#define DNF_INLIST    0x0001


/** The horizontal distance of the frame to the text in an arc label. */
#define ARCLABEL_FRAME_OFFS_X  3


/** The vertical distance of the frame to the text in an arc label. */
#define ARCLABEL_FRAME_OFFS_Y  3


/** The Canvas class provides all functionality to display and modify nodes in Grisu. */
class Canvas:public QWidget{
  Q_OBJECT
private:

  
  /** This structure represents a hit region for mouse hit testing. */
  struct KlickBox {
    /** The address of the node at this position. */
    NodeAddress addr;

    
    /** The bounding rectangle of the node. */
    QRect box;

    
    /** The type of the node (one of the <code>NODETYPE_xxxx</code> constants). */
    int type;
  };


  /** The local (window-specifc) copy of the application options. This allows users to
      change some options without affecting other windows. */
  Options *options;


  /** The node the user clicked on last. */
  KlickBox *klickedOn;


  /** All click regions are kept in this list to allow mapping between click regions
      and nodes. */
  QList<KlickBox> toKlick;


  /** The painter object. */
  QPainter *paint;


  /** The total size of a structure or a tree. */ 
  QSize maxSize;


  /** The current X offset to the top left corner of the structure. */
  int offsetX;


  /** The current Y offset to the top left corner of the structure. */
  int offsetY;


  /** The old X offset (used during dragging of a structure). */
  int oldOffsetX;


  /** The old Y offset (used during dragging of a structure). */
  int oldOffsetY;


  /** The current X coordinate of the mouse cursor during dragging. */
  int dragX;


  /** The current Y coordinate of the mouse cursor during dragging. */
  int dragY;


  /** Determines whether nodes should actually drawn or not.
      This way, the actual drawing routines can be used to determine
      the size of a node (and all its subnodes), without actually
      drawing the node. */
  bool realDraw; 


  /** The root node of the structure or the tree. */
  GenericNode *rootNode;


  /** A pointer to the document containing the node data. */
  Document *document;


  /** Pointer to the vertical scrollbar. Note: The scrollbar is managed
      by the parent widget. */
  MyScrollBar* v_scroll;


  /** Pointer to the horizontal scrollbar. Note: The scrollbar is managed
      by the parent widget. */
  MyScrollBar* h_scroll;
  

  /** Pointer to the popup menu currently open (or <code>NULL</code> if there
      is none). */
  KPopupMenu *popup;


  /** <code>true</code> if the user is currently dragging the structure or tree
      with the mouse. */
  bool dragging;


  /** Pointer to the currently highlighted node. */
  DisplayableNode* highlight;


  /** Pointer to the node that is currently marked by the finder. */
  DisplayableNode* foundMark;


  /** A search tree of node addresses. */
  IdTree* dispNodes;


  /** Draws a rectangle around a node indicated that it was found in a search
      operation.
      @param x the X coordinate of the rectangle
      @param y the Y coordinate of the rectangle
      @param w the width of the rectangle
      @param h the height of the rectangle */
  void drawFoundMark(int x, int y, int w, int h);


  /** Returns a list that contains the type and all attributes of a
      feature structure (a <code>StrucNode</code>).
      @param node the <code>StrucNode</code>
      @return a list containing the type as its first element followed by all
        attributes. */
  QStringList typeAndFeatsList(StrucNode* node);


  /** Returns a list that contains the names of all hidden child nodes. Only works for
      nodes of type StrucNode, where all hidden attributes are returned.
      @param addr the node address of the parent node.
      @return a list of all hidden node names. This list is empty if no hidden child nodes exist or
      <code>dn</code> contains a node with an unsupported node type. */
  QStringList namesOfHiddenChildNodes(NodeAddress& addr);


 public:
  /** Creates a new canvas that displays a structure or a tree. */
  Canvas(Options *, MyScrollBar *, MyScrollBar *,
	 Document *, GenericNode* rootNode , QWidget *parent=0, const char *name=0 );


  /** Destroys the canvas. */
  ~Canvas();


  /** Called when the window needs to be painted. */
  virtual void paintEvent(QPaintEvent *);


  /** Called when the window needs to be repainted. */
  virtual void repaint(bool erase=true);


  /** Called when the window needs to be painted. */
  virtual void repaint(int, int, int, int, bool erase=true);


  /** Prints the current structure or tree. */
  void printToPrinter(QPrinter &p);


  /** Calculates the dimensions of a given node (by executing the drawing routines with
      <code>realDraw</code> set to <code>false</code>).
      @param nd the node
      @param addrParent the node address of the parent node
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node.
      @return the width and height of the node. */
  QSize getNodeDimensions(GenericNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a node by delegating the work to be done to one of the specialized drawing routines
      depending on the node's type. Drawing is done recursively, top to bottom, left to right.
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @param flags additional flags (a combination of <code>DNF_xxxx</code> constants).
      @return the width and height of the node. */
  QSize drawGenericNode(GenericNode* nd, const NodeAddress& addrParent, int dx, int dy, unsigned short flags = 0);


  /** Draws a placeholder of a hidden node. 
      @param dnd the displayable node to be drawn
      @param text the text that appears as a placeholder
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawHiddenNode(DisplayableNode* dnd, char* text, int dx, int dy);


  /** Draws a type node.
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawTypeNode(TypeNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a feature and its value.
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawFeatvalNode(FeatvalNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a list node.
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawListNode(ListNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a set node. 
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawSetNode(SetNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a feature structure.
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawStrucNode(StrucNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a function node. 
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawFunctionNode(FunctionNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a relation node.
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawRelationNode(RelationNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a tag (a number in a little box).
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @param flags additional flags (a combination of <code>DNF_xxxx</code> constants).
      @return the width and height of the node. */
  QSize drawRefNode(RefNode* nd, const NodeAddress& addrParent, int dx, int dy, unsigned short flags = 0);


  /** Draws an atom node.
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawAtomNode(AtomNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a tree node.
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawTreeNode(TreeNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a tail node.
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawTailNode(TailNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a rest node. 
      @param nd the node to be drawn
      @param addrParent the node address of the parent node.
      @param dx the X coordinate of the node
      @param dy the Y coordinate of the node
      @return the width and height of the node. */
  QSize drawRestNode(RestNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a DisjunctionNode. DisjunctionNodes represent the infix disjunction operator.
      @param nd the DisjunctionNode to be drawn
      @param addrParent the unique node address of the displayed node
      @param dx the x coordinate of the disjunction
      @param dy the y coordinate of the disjunction
      @return the size of the complete node (the extent of the operators and all operands). */
  QSize drawDisjunctionNode(DisjunctionNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Draws a ConjunctionNode. ConjunctionNodes represent the infix conjunction operator.
      @param nd the ConjunctionNode to be drawn
      @param addrParent the unique node address of the displayed node
      @param dx the x coordinate of the conjunction
      @param dy the y coordinate of the conjunction
      @return the size of the complete node (the extent of the operators and all operands). */
  QSize drawConjunctionNode(ConjunctionNode* nd, const NodeAddress& addrParent, int dx, int dy);


  /** Selects a new root node.
      @param node the node designated to be the new root node. */
  void setRootNode(GenericNode* node);


  /** Updates the options in this canvas to match the application wide options.
      This is called when the user changes a font, for example.
      @param op a reference to the application wide options object. */
  void updateOptions(const Options& op);


  /** Adds a click region to the list of click regions.
      @param addr the address of the node in this region
      @param nodetype the type of the node
      @param bounds the coordinates of the click region. */
  void addToClick(const NodeAddress& addr, int nodetype, QRect bounds);


  /** Toggles the hide flag on a displayable node.
      @param nd the displayable node
      @param redraw indicates whether the structure should be redrawn immediately. */
  void hideUnhide(DisplayableNode* nd, bool redraw=true);


  /** Toggles the expanded flag on a displayable node.
      @param nd the displayable node
      @param redraw indicates whether the structure should be redrawn immediately. */
  void expandCollapse(DisplayableNode* nd, bool redraw = true);


  /** Updates the scrollbars after the window has been resized or the data has changed. */
  void resetScrollbar();


  /** Returns the total size of the currently displayed structure or tree. */
  QSize getMaxSize() {return maxSize;};


  /** Returns the current X offset. */
  int getOffsetX() {return offsetX;}


  /** Returns the current X offset. */
  int getOffsetY() {return offsetY;}


  /** Returns the click region of the node last clicked on. */
  KlickBox* getKlickedOn();


  /** Returns the click region which contains the given point, if any.
      @param p the point
      @return a pointer to the click region (the node) that contains the point, or
        <code>NULL</code> of no such region exists. */
  KlickBox* whereKlicked(const QPoint &p);
  

  /** Returns the tree of node ids */
  IdTree* getDispNodes(void);


  /** Resets the currently displayed structure or tree. 
      Everything is reset to the initial state. */
  void reset();

  
  /** Make sure that the given node is visible.
      @param addr the address of the node.  */
  void ensureVisible(const NodeAddress* addr);


  /** Move the found marker to a new node and scroll to that position.
      @param addr the address of the node to be marked as found. */
  void markFound(const NodeAddress* addr);


  /** Returns the document displayed in this Canvas.
      @return the document. */
  Document* getDocument(void) const;
  

protected:
  /** Called when the user clicks with the mouse. */
  void mousePressEvent(QMouseEvent *);


  /** Called when the user double clicks with the mouse. */
  void mouseDoubleClickEvent(QMouseEvent *mo);


  /** Called when the user moves the mouse. */
  void mouseMoveEvent(QMouseEvent *);


  /** Called when the user releases a mouse button. */
  void mouseReleaseEvent(QMouseEvent *);


  /** Called when a wheel event occurs (wheel mice only).
      @param we the wheel event class. */
  virtual void wheelEvent(QWheelEvent* we);



private:
  /** Draws text if <code>realDraw</code> is set to <code>true</code>. */
  void   drawText(int, int, const char *, int width=0);
  

  /** Draws a line if <code>realDraw</code> is set to <code>true</code>. */
  void   drawLine(int, int, int, int);


  /** Draws a rectangle if <code>realDraw</code> is set to <code>true</code>. */
  void   drawRect(int, int, int, int);

  
  /** Draws an AND operator, but only if <code>realDraw</code> is set.
      @param x the left X coordinate of the operator
      @param width the total width of the operator (should be divisble by 2 to make sure the operator
                   is symmetric)
      @param yu the Y coordinate of the "peak" of the operator
      @param yd the Y coordinate of the baseline of the operator. */
  void drawAndOperator(int x, int width, int yu, int yd);


  /** Draws an OR operator, but only if <code>realDraw</code> is set.
      @param x the left X coordinate of the operator
      @param width the total width of the operator (should be divisble by 2 to make sure the operator
                   is symmetric)
      @param yu the Y coordinate of the "peak" of the operator
      @param yd the Y coordinate of the baseline of the operator. */
  void drawOrOperator(int x, int width, int yu, int yd);


  /** Clears the list of current click regions. */
  void resetToClick(void);
  
  
  /** Returns the top left coordinate of a node.
      @param addr the address of the node. */
  QPoint nodeOrigin(const NodeAddress* addr);

private slots:
  /** Collapses or expands the node the user clicked on (if this operation is supported 
      for the type of the node). */
  void expandCollapseSlot();    


public slots:
  /** Called when the user is scrolling. */
  void scrollingSlot(int);


  /** Scrolls one line down. */
  void scrollLineDownSlot(void);


  /** Scrolls one line up. */
  void scrollLineUpSlot(void); 


  /** Scrolls one line right. */
  void scrollLineRightSlot(void); 


  /** Scrolls one line left. */
  void scrollLineLeftSlot(void); 

  
  /** Scrolls one page down. */
  void scrollPageDownSlot(void);


  /** Scrolls one page up. */
  void scrollPageUpSlot(void); 


  /** Scrolls one page right. */
  void scrollPageRightSlot(void); 


  /** Scrolls one page left. */
  void scrollPageLeftSlot(void); 


  /** Hides all placeholders of hidden attributes and the type
      of a feature structure. */
  void hideLevelPopupSlot(void);


  /** Shows all placeholders of hidden attributes and the type
      of a feature structure. */
  void unhideLevelPopupSlot(void);


  /** Hides a node. */
  void hideNodePopupSlot(void);


signals:
  /** Emitted when the canvas needs to be redrawn. */
  void redrawSignal();
};

#endif
