/*  $Id: data.h,v 1.29 2003/05/22 16:43:10 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file
    
    This file contains the declarations for the Document class that contains all node data.

    @author Holger Wunsch */


#ifndef __DATA_H_
#define __DATA_H_

#include "node.h"
#include <qobject.h>
#include <qstring.h>
#include <qstrlist.h>
#include <qtstream.h>

#define DRAW_AUTO 0   /**< Automatically determine whether to draw a tree or a feature structure based on the
			   data available (if tree data is available, draw a tree, otherwise a structure) */
#define DRAW_STRUC 1  /**< Draw a structure when initially displaying the data. */
#define DRAW_TREE 2   /**< Draw a tree when initially displaying the data. */


/** The Document class manages all node data. It provides several accessor methods to different parts of
    the data (main feature structure, tree structure, etc.). Furthermore, it maintains a mapping between 
    node IDs and corresponding nodes.

    @author Holger Wunsch */
class Document {
private:
  /** The name of this document. This name appears on the titlebar in a view window that displays this
      document. */
  QString name;

public:
  /** A list that contains a ll structure nodes (that is all nodes except trees, relations and reentrancies).<br><br>

      <b>Note:</b>The first element on this list must be the root node of the main feature structure. */
  QList<GenericNode>*  data; 

  /** A list of all ReentNodes in the data. This list may be empty. */
  QList<ReentNode> *Reent;
  
  /** Pointer to the root node of the tree structure, or <code>NULL</code>, if no tree exists. */
  TreeNode* tree;

  /** A list of all RelationNodes in the data. This list may be empty. */
  QList<RelationNode> *rel;

  /** A NodeIdMap that maps node IDs to the corresponding nodes. */
  NodeIdMap* map;
  
  /** Determines how this document should be displayed when a view window is opened, one of the <code>DRAW_xxxx</code>
      constants. */
  int showDefault;

  /** Creates a new instance of this class.
      @param s the name of this document. Appears on the titlebar of a view window that displays this document. */
  Document(const QString& s);

  /** Deletes all node data and the instance. */
  ~Document();

  /** Initializes the document with the data built by the parser.
      @param newData a List of all nodes. Nodes are automatically sorted into the data, Reent, tree and rel data members.
      @param m the NodeIdMap that maps node IDs to the corresponding nodes. */
  void setData(List* newData, NodeIdMap* m);

  /** Returns the ReentNode with the given reference ID.
      @return the corresponding ReentNode or <code>NULL</code> if no ReentNode was found. */
  ReentNode* findReentrancy(int ref);

  /** Returns a node given its ID.
      @param id the node ID.
      @return the node, or <code>NULL</code> if no node with this ID was found. */
  GenericNode* findNodeById(char* id);

  
  /** Returns the highest node ID in the document. */
  unsigned int maxID(void);

  
  /** Returns the name of the document. The name appears on the titlebar of a view window that displays this document.
      @return the document's name. */
  QString getName();

  /** Writes the document to a file. Only the node data as it is sent by the backend is written to the file, not
      any modifications users applied (that is, DisplayableNode properties are not saved).
      @param file a file handle of a file opened with write access. */
  void write(FILE* file);
};


#endif
