/*  $Id: gbuffer.cpp,v 1.12 2003/05/22 16:43:11 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @file

   Implementation of the GBuffer class.

   @author Holger Wunsch */


#include <string.h>
#include "gbuffer.h"


/** Creates a new GBuffer with a given initial site.
    @param size the initial size of the buffer.
    @param growDelta the amount by which the buffer grows if more space
    is needed. */
GBuffer::GBuffer(unsigned int size, unsigned int growDelta) {
  maxSize = size;

  data = new unsigned char[maxSize];

  this->size = 0;
  pos = 0;
  this->growDelta = growDelta;
}


/** Destroys the buffer and the data in it. */
GBuffer::~GBuffer(void) {
  delete [] data;
}


/** Makes sure that data of the size given can be inserted into the
    buffer without a buffer overflow.
    
    There are two strategies to achieve this:
    If the distance of the current position (the read position) is greater
    than the new size needed, the contents in the buffer up to the current 
    position are deleted and all subsequent data is moved to the beginning
    of the buffer.
    
    Otherwise, a larger memory block is allocated.
    @param size the size of the data to be inserted into the buffer. */
void GBuffer::ensureSize(unsigned int size) {
  if (this->size + size > maxSize) {
    // First case: There is a block of obsolete data (data that has
    // already been read) of sufficient size at the beginning.
    // Move all unread data to the beginning of the buffer, which
    // makes room for the new data at the end of the buffer
    
    if (pos >= size) {
      memcpy(data, &(data[pos]), this->size - pos);
      this->size -= pos;
      pos = 0;
    }
    // Second case: The data block at the beginning is not big enough.
    // Allocate more space.
    else {
      if (size <= growDelta)
	maxSize += growDelta;
      else
	maxSize += size;

      unsigned char* newData = new unsigned char[maxSize];
      memcpy(newData, data, this->size);
      
      delete [] data;
    
      data = newData;
    }
  }
}


/** Adds data to the buffer at the end of the buffer. Necessary space 
    is allocated automatically.
    @param data pointer to the data to be added
    @param size the size of the data. */
void GBuffer::add(unsigned char* data, unsigned int size) {
  ensureSize(size);

  memcpy(&(this->data[this->size]), data, size);
  
  this->size += size;
}


/** Returns a pointer to the end of the buffer which can be used in
    library routines that require a buffer as a target (<code>read(...)</code>)
    for example, ensuring that enough space is left in the buffer.
    
    <b>Handle with care!</b> It is up to the caller to make sure that
    no more data than <code>size</code> is added to the buffer!
    
    After adding data to the buffer with an external function, <code>added(...)</code>
    <b>must</b> be called, otherwise the data just added is overwritten in the
    next call of <code>add</code> or <code>addPtr</code>!
    @param size the maximum size of data to be copied to the buffer
    @return a pointer to the end of the buffer. */
unsigned char* GBuffer::addPtr(unsigned int size) {
  ensureSize(size);

  return &(data[this->size]);
}


/** Moves the end of the buffer by <code>size</code> bytes.
    
   <b>Must</b> and <b>must only</b> be used after data was added to
   the buffer using an external function like the C library's 
   <code>read(...)</code> in conjunction with <code>addPtr(...)</code>.
   @param size the number of bytes the buffer end is to be moved. */
void GBuffer::added(unsigned int size) {
  this->size += size;
}


/** Stores up to <code>size</code> bytes of data starting at the
    current position in a target buffer <code>buffer</code>. The
    current position is increased by the amount of data read.
    Note that the data remains unchanged in the buffer.
    @param buffer the target buffer, which must at least be of
      size <code>size</code>.
    @param size the size of the target buffer.
    @return the number of bytes copied to the target buffer. */
unsigned int GBuffer::get(unsigned char* buffer, unsigned int size) {
  unsigned int copySize;

  if (this->size - pos >= size)
    copySize = size;
  else
    copySize = this->size - pos;

  memcpy(buffer, &(data[pos]), copySize);

  pos += copySize;

  return copySize;
}


/** Returns a pointer to the current position. */
const unsigned char* GBuffer::getPtr(void) const {
  return &(data[pos]);
}


/** Returns a line of text, starting at the current position, ending
    at the next newline character, in a string. If no newline character
    can be found up to the end of the buffer, <code>NULL</code> is
    returned. The current position is increased by the amount of data read.
    @return the line of text (terminated with a newline character
      <code>\n</code>), or <code>NULL</code> if no newline character
      can be found up to the end of the buffer. <b>Note</b>: It is up
      to the caller to property free the string when it is not needed
      any more. */
unsigned char* GBuffer::readLine(void) {
  unsigned char* newlinePtr = (unsigned char*) memchr(&(data[pos]), '\n', size - pos);

  if (newlinePtr) {
    unsigned int newlinePos = newlinePtr - data; 
    unsigned int length = newlinePos - pos + 1;
    unsigned char* line = new unsigned char[length + 1]; // One more byte for 0x00

    memcpy(line, &(data[pos]), length);
    line[length] = 0x00;

    pos = newlinePos + 1;

    return line;
  }

  return NULL;
}


/** Reallocates the memory block occupied by the buffer such that
    the maximal size is equal to the current size, for more efficient
    memory usage. */
void GBuffer::shrink(void) {
  if (maxSize > size) {
    maxSize = size;

    unsigned char* newData = new unsigned char[maxSize];

    memcpy(newData, data, size);

    delete [] data;

    data = newData;
  }
}


/** Returns the size of the data currently stored in the buffer.
    @return the size of the data. */
unsigned int GBuffer::getSize(void) {
  return size;
}
