/*  $Id: gdataexchange.cpp,v 1.12 2003/05/22 16:43:11 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    Implementation of the GDataExchange class.

    @author Holger Wunsch */


#include <sys/socket.h>
#include <netdb.h>
#include <qmessagebox.h>
#include <unistd.h>
#include <stdio.h>
#ifdef _SOLARIS
#include <sys/filio.h>
#endif
#ifdef _LINUX
#include <sys/ioctl.h>
#endif

#include "gdataexchange.h"

#include "gdataexchange.moc"


/** Creates a new data exchange object. Sets up the server socket.
    @param reader the data reader that handles incoming data. */
GDataExchange::GDataExchange(GDataReader* reader) {
  inData = NULL;
  outData = NULL;

  dataReader = reader;

  serverSocket = NULL;
  socket = NULL;
}


/** Destroys the data exchange object. */
GDataExchange::~GDataExchange(void) {
  closeServerSocket();
}


/** Finds a free port in the range <code>GDE_PORTRANGE_LO</code> and 
    <code>GDE_PORTRANGE_HI</code> and creates a server socket listening on this
    port. Raises a <code>connectionStateSignal</code> with information
    about the status of the connection, and also prints the status information
    to <code>stdout</code>.
    @return <code>true</code> if the server socket was set up successfully,
      <code>false</code> otherwise. */
bool GDataExchange::setupServerSocket(void) {
  port = GDE_PORTRANGE_LO;

  // Scan the port range for a free port
  while (true) {
    serverSocket = new KServerSocket(port);

    if (serverSocket->socket() <= -1) {
      // The port just tried is occupied
      delete serverSocket;
      
      if (port == GDE_PORTRANGE_HI) {
	fprintf(stderr, "Opening of the socket connection failed.\n");
	serverSocket == NULL;
	emit connectionStateSignal(GDE_STATE_NO_CONNECTION, NULL, -1);
	return false;
      }
    }
    else
      break;

    port++;
  }


  gethostname(hostname, GDE_HOSTNAME_MAXLEN);
  
  hostent* hostEntry = gethostbyname(hostname);
  printf("@|@| %s %d\n", hostEntry->h_name, port);
  fflush(stdout);

  emit connectionStateSignal(GDE_STATE_READY, hostname, port);

  connect(serverSocket, SIGNAL(accepted(KSocket*)), SLOT(acceptedSlot(KSocket*)));

  socket = NULL;

  return true;
}


/** Closes the server socket. */
void GDataExchange::closeServerSocket(void) {
  if (socket)
    closeSlot(socket);

  if (serverSocket)
    delete serverSocket;

  emit connectionStateSignal(GDE_STATE_NO_CONNECTION, NULL, -1);
}


/** Called when the server socket accepted a connection.
    @param sock the socket that was created after accepting the connection. */
void GDataExchange::acceptedSlot(KSocket* sock) {
  if (socket)
    delete socket;

  socket = sock;
  int flag = 1;
  if (ioctl(socket->socket(), FIONBIO, &flag) == -1)
    fprintf(stderr, "Non-blocking mode can't be set for socket.\n");

  inData = new GBuffer(GDE_INBOUND_BUFFER_SIZE, GDE_INBOUND_BUFFER_GROW_DELTA);

  outData = new GBuffer(GDE_OUTBOUND_BUFFER_SIZE);

  socket->enableRead(true);

  QObject::connect(socket, SIGNAL(readEvent(KSocket*)), this, SLOT(readSlot(KSocket*)));
  QObject::connect(socket, SIGNAL(writeEvent(KSocket*)), this, SLOT(writeSlot(KSocket*)));
  QObject::connect(socket, SIGNAL(closeEvent(KSocket*)), this, SLOT(closeSlot(KSocket*)));

  emit connectionStateSignal(GDE_STATE_CONNECTED, hostname, port);
}


/** Asynchronously sends out data.
    @param data the data to be sent.
    @param size the size of the data. */
void GDataExchange::asyncSend(unsigned char* data, unsigned int size) {
  if (socket) {
    outData->add(data, size);
    
    socket->enableWrite(true);
  }
}


/** Synchronously sends out data. Does not return before the data has
    been sent. This method performs the same functionality as
    <code>write(...)</code> from the C library.
    @param data the data to be sent
    @param size the size of the data
    @return the number of bytes written, or <code>-1 (0xFFFFFFFF)</code> in case of
    an error (<code>errno</code> can be queried to get additional
    information since this function directly calls <code>write(...)</code>). */
unsigned int GDataExchange::syncSend(unsigned char* data, unsigned int size) {
  if (socket)
    return (unsigned int) write(socket->socket(), data, size);
  else
    return (unsigned int) -1;
}


/** Synchronously reads data. If no data is available, this method immediately returns
    (non-blocking I/O).
    @param buffer the buffer the data is to be stored in
    @param size the size of the buffer.
    @return the number of bytes read, or <code>-1 (0xFFFFFFFF)</code> if no data
    is available or in case of an error (<code>errno</code> can be queried to get
    additional information since this function directly calls <code>read(...)</code>). */
unsigned int GDataExchange::syncRead(unsigned char* buffer, unsigned int size) {
  if (socket)
    return (unsigned int) read(socket->socket(), buffer, size);
  else
    return (unsigned int) -1;
}


/** Called when data is available for reading. After all available data was read, 
    it is passed on to <code>dataReader->processData</code>.
    @param sock not used. */
void GDataExchange::readSlot(KSocket* sock) {
  int sockHandle;
  int bytesRead;
  unsigned char buffer[64000];

  sockHandle = socket->socket();

  do {
    bytesRead = read(sockHandle, buffer, 64000);
    if (bytesRead > 0)
      inData->add(buffer, bytesRead);
  } while (bytesRead > 0);

  dataReader->processData(inData);
}


/** Called when the socket is ready for writing. Writes all data on <code>outBuffer</code>
    to the socket, then deletes the buffer.
    @param sock not used. */
void GDataExchange::writeSlot(KSocket* sock) {
  int sockHandle;
    
  socket->enableWrite(false);

  sockHandle = socket->socket();

  write(sockHandle, outData->getPtr(), outData->getSize());

  delete outData;
  outData = new GBuffer(GDE_OUTBOUND_BUFFER_SIZE);
}


/** Called when the socket connection was closed.
    @param sock not used. */
void GDataExchange::closeSlot(KSocket* sock) {
  delete socket;
  socket = NULL;

  delete inData;
  inData = NULL;

  delete outData;
  outData = NULL;

  emit connectionStateSignal(GDE_STATE_READY, hostname, port);
}
