/*  $Id: glatexrenderer.h,v 1.5 2003/05/22 16:43:11 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file

    Definition of the <code>GLatexRenderer</code> class.

    @author Holger Wunsch */


#ifndef __GLATEXRENDERER_H
#define __GLATEXRENDERER_H


#include <stdio.h>

#include <qlist.h>
#include <qstring.h>

#include "data.h"
#include "node.h"
#include "NodeAddress.h"
#include "idtree.h"


/** The maximum size of the buffer used in <code>GLatexRenderer::latexifyString</code>. */
#define LATEXIFY_BUFFER_SIZE  2000


/** This class generates LaTeX-Output from the node data. 

    @author Holger Wunsch */
class GLatexRenderer {
  /** The data */
  Document* data;


  /** The displayable node data. */
  IdTree* dispNodes;


  /** Name of the output file. */
  QString outName;


  /** Handle to the output file (<code>NULL</code> if no file opened) */
  FILE* outFile;


  /** Indicates how the data should be displayed. The value must be one of 
      the <code>DRAW_xxx</code> constants. */
  int drawMode;


  /** Renders a generic node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderGenericNode(GenericNode* a_node, const NodeAddress& a_addrParent);


  /** Renders an atom node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderAtomNode(AtomNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a conjunction node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderConjunctionNode(ConjunctionNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a disjunction node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderDisjunctionNode(DisjunctionNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a featval node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderFeatvalNode(FeatvalNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a function node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderFunctionNode(FunctionNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a list node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderListNode(ListNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a reent node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderReentNode(ReentNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a ref node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderRefNode(RefNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a rest node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderRestNode(RestNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a set node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderSetNode(SetNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a struc node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderStrucNode(StrucNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a tail node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderTailNode(TailNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a tree node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderTreeNode(TreeNode* a_node, const NodeAddress& a_addrParent);


  /** Renders a type node.
      @param a_node the node.
      @param a_addrParent the node address of the parent node.
      @return <code>true</code> if the node was written, <code>false</code> if not,
      for example because the node is hidden. */
  bool renderTypeNode(TypeNode* a_node, const NodeAddress& a_addrParent);


  /** Writes the LaTeX header to the output file.
      @return <code>true</code> if the header was written successfully,
      <code>false</code> otherwise. */
  bool writeLatexHeader(void);


  /** Writes the necessary LaTeX closing statements to the output file. 
      @return <code>true</code> if the header was written successfully,
      <code>false</code> otherwise. */
  bool writeLatexFooter(void);


  /** Writes the opening statements of an <code>avm</code> environment.
      @return <code>true</code> if the statement was written sucessfully, <code>false</code>
      otherwise. */
  bool writeAVMHeader(void);


  /** Writes the closing statements of an <code>avm</code> environment.
      @return <code>true</code> if the statement was written sucessfully, <code>false</code>
      otherwise. */
  bool writeAVMFooter(void);


  /** Modifies a string such that it prints properly in LaTeX.
      
      In LaTeX, several characters are command characters which must be
      escaped to make them print properly. These charaters are:
      
      <pre>$, %, {, }, &, #, _, ^</pre>
      
      @param a_inStr the string to be modified
      @return a pointer to a static buffer which contains the modified string. */
  char* latexifyString(const char* a_inStr);


 public:
  /** Creates a LaTeX renderer.
      @param a_data the data
      @param a_drawMode either <code>DRAW_STRUC</code>, <code>DRAW_TREE</code>, or
      <code>DRAW_AUTO</code>. If <code>DRAW_AUTO</code> is given, tree output is generated
      when tree data is available, otherwise the data is rendered as feature structure.
      @param a_dispNodes the displayable node data
      @param a_outName the name of the output file. */
  GLatexRenderer(Document* a_data, int a_drawMode, IdTree* a_dispNodes, const QString& a_outName);

  
  /** Destroys this instance. */
  ~GLatexRenderer(void);


  /** Renders the node data.
      @return <code>true</code> if the rendering process was successful,
      <code>false</code> if an error occurred. */
  bool render(void);
};


#endif
