/*  $Id: idtree.h,v 1.29 2003/05/22 16:43:12 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file
    Declarations for the <code>IdTree</code> class.
    @author Holger Wunsch */


#ifndef __IDTREE_H
#define __IDTREE_H

#include "dispnode.h"


/** One mapping between a node address and a displayable node.

    Remark: The node address is contained in the <code>DisplayableNode</code>.

    @author Holger Wunsch */
struct NodeAddrMapping {
  /** The displayable node. */
  DisplayableNode* node;


  /** The next mapping, or <code>NULL</code> if no more mappings are in the list. */
  NodeAddrMapping* next;
};


/** Pointer to <code>NodeAddrMapping</code>. */
typedef NodeAddrMapping* NodeAddrMappingPtr;


/** This class implements a fast mapping between node addresses and displayable nodes. In earlier
    versions, this class was implemented as a tree. Although this is no longer true, the name
    is kept for compatibility.

    All nodes that are displayed by Grisu (which are all instances of DisplayableNode),
    are assigned a unique node address. The node address is composed of the concatenation
    of the node ids from the root node of a feature structure or a tree to the node id of
    the current node (see also the explanation of NodeAddress).

    Node addresses are accessed very frequently, especially during painting, where for each
    node its address must be found at least once. 

    The basic idea used in this implementation is very similar to a hash table.
    The mapping makes use of the fact that although many nodes with different node addresses
    occur in a structure, individual nodes don't occur that often (less than ten times). So
    the search space can be restricted significantly by looking at the first component in a
    node address, which is the ID of the node the node address points to. This class maintains
    an array that contains as many elements as the largest node ID in the structure displayed.
    (You can think of the ID as the hash key).
    The array elements point to lists of node addresses that all start with this node ID. If
    there are multiple node addresses that start with this ID, the right one must be found
    using a string comparison of the node address.

    @author Holger Wunsch */
class IdTree {
  /** The maximum number of mappings. */
  unsigned int maxMappings;


  /** An array with elements that are lists of known node addresses that start with a node ID
      that is equal to the element's array index. */
  NodeAddrMappingPtr* addrs;

public:
  /** Creates a new, empty map that can hold up to <code>max</code> mappings.
      @param max the maximum number of mappings. */
  IdTree(unsigned int max);

  
  /** Destroys the entire map. */
  ~IdTree(void);


  /** Adds a node to the map. If the node has been added previously,
      its corresponding DisplayableNode is returned. 
      @param na the node address of the node
      @param node a pointer to the node data
      @return a DisplayableNode representing the visible instance
      of the node. */
  DisplayableNode* addDisplayableNode(const NodeAddress& na, GenericNode* node);


  /** Looks up a DisplayableNode in the map given the node address of
      the corresponding node.
      @param na the node address
      @return The DisplayableNode or <code>NULL</code> if the no node wirh the
      given address exists in the id tree. */
  DisplayableNode* findDisplayableNode(const NodeAddress& na);
};

#endif
