/*  $Id: mainw.h,v 1.33 2003/05/22 16:43:13 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file

    Contains the declarations for the MainWindow class.

    @author Holger Wunsch */

#ifndef __MAINWINDOW_H
#define __MAINWINDOW_H


#include <kstatusbar.h>
#include <kapp.h>
#include <kmainwindow.h>
#include <kmenubar.h>
#include <kiconloader.h>
#include <kapp.h>
#include <ksock.h>
#include <qpopmenu.h>
#include <kstatusbar.h>
#include <ktoolbar.h>
#include <qlistbox.h>

#include "data.h"
#include "vcanvas.h"
#include "gdatareader.h"


/* Menuitem - IDs */
#define MID_FILE_SAVE                101   /**< Menuitem ID */
#define MID_DATA_SHOW_TREE           201   /**< Menuitem ID */
#define MID_DATA_SHOW_STRUC          202   /**< Menuitem ID */
#define MID_DATA_COMPARE_STRUC       203   /**< Menuitem ID */
#define MID_DATA_COMPARE_TREE        204   /**< Menuitem ID */
#define MID_DATA_DELETE_DATA         205   /**< Menuitem ID */
#define MID_DATA_DELETE_ALL_DATA     206   /**< Menuitem ID */
#define MID_OPTIONS_EXPAND_REENT     301   /**< Menuitem ID */
#define MID_OPTIONS_FIT              302   /**< Menuitem ID */
#define MID_OPTIONS_HIGHLIGHT        303   /**< Menuitem ID */
#define MID_OPTIONS_AUTOOPEN         304   /**< Menuitem ID */
#define MID_WINDOW_CLOSEALL          401   /**< Menuitem ID */
#define MID_ABOUT_GRISU              501   /**< Menuitem ID */
#define MID_ABOUT_KDE                502   /**< Menuitem ID */


/* Toolbar - IDs */
#define TID_SEPARATOR               0      /**< Toolbar ID of separators */
#define TID_FILE_OPEN               1      /**< Toolbar ID */
#define TID_FILE_SAVE               2      /**< Toolbar ID */
#define TID_DATA_DELETE_DATA        3      /**< Toolbar ID */
#define TID_DATA_SHOW_STRUC         4      /**< Toolbar ID */
#define TID_DATA_SHOW_TREE          5      /**< Toolbar ID */
#define TID_WINDOW_CLOSEALL         6      /**< Toolbar ID */


/* Statusbar - IDs */
#define SID_CONNECTION_STATE        0      /**< Statusbar ID connection state */
#define SID_CONNECTION_HOST         1      /**< Statusbar ID hostname and port */


/** The MainWindow class implements Grisu's main window. All user interactions with the
    main window are handled in this class.

    @author Holger Wunsch */
class MainWindow : public KMainWindow, public GDataReader
{
  Q_OBJECT

  /** The list of open view windows (instances of ViewCanvas). */
  QList<ViewCanvas>* views;
  

  /** This is set to <code>true</code> while executing a "Close All Views" command. */
  bool closingAll;


  /** The name of the host Grisu is running on. */
  char hostname[500];


  /** The port number which is used for the socket connection. */
  int port;


  /** Compares two structures or trees.
      @warning The built-in structure comparison algorithm is currently incomplete.
      See comapre.cpp. */
  void compareStructuresOrTrees(int what);


  /** Deletes all data and closes associated views, if necessary.
      @param selOnly If <code>selOnly</code> is <code>true</code>, only selected items in the data list
      will be deleted, otherwise all items will be deleted. */
  void deleteData(bool selOnly);


public:

  /** Initializes the main window. Creates the menu and the toolbar, connects menu and toolbar items to
      slots and initializes the socket connection by calling MainWindow#initializeSocket.
      @param parent the parent widget
      @param name the window name (appears in the title bar) */
  MainWindow( QWidget *parent =0, const char *name=0);


  /** Deletes the instance and all its data members and closes the socket connection. */
  ~MainWindow();


  /** Updates the list box. */
  void updateListBox();


  /** Opens a new view window for the given document.
      @param d the document to be shown
      @param what The way how the data should be displayed, either as a structure (<code>DRAW_STRUC</code>), or
      as a tree (<code>DRAW_TREE</code>).
      @param x the X coordinate of the window
      @param y the Y coordinate of the window
      @param w the width of the window
      @param h the height of the window
      @return a pointer to the new view window widget. */
  ViewCanvas* newViewWindow(Document* d, int what, int x = -1, int y = -1, int w = 480, int h = 480);


  /** Loads a GRISU data file and creates a new view window, if the user chose to automatically open
      view windows. 
      @param fileName the name of the data file. */
  void loadDataFile(const char* fileName);


  /** This method is called by the data exchange manager when new data has arrived. */
  virtual void processData(GBuffer* buffer);


protected:
 
  /** Pointer to the main menu. */
  KMenuBar *mainMenu;

  
  /** Pointer to the File menu. */
  QPopupMenu *fileMenu;


  /** Pointer to the Data menu. */
  QPopupMenu *dataMenu;

  
  /** Pointer to the help (Info) menu. */
  QPopupMenu* helpMenu;


  /** Pointer to the Options menu. */
  QPopupMenu* optionMenu;


  /** Pointer to the Window menu. */
  QPopupMenu* windowMenu;


  /** Pointer to the Options|Colors menu. */
  QPopupMenu* colorMenu;


  /** Pointer to the Options|Font menu. */
  QPopupMenu* fontMenu;


  /** Pointer to the status bar. */
  KStatusBar *statusBar;


  /** Pointer to the toolbar. */
  KToolBar *mainToolBar;


  /** The listbox widget that displays the list of data packages. */
  QListBox *listBox;


public slots:

  /** Opens a file. */
  void slotLoad();
  

  /** Saves the currently displayed structure. */
  void slotSave();

  
  /** Shows the current data as a structure. */
  void showStructureSlot();


  /** Shows the current data as a tree. */
  void showTreeSlot();


  /** Handles double clicks. */
  void doubleKlickSlot(int);


  /** Deletes the currently selected data package. */
  void deleteDataSlot();


  /** Deletes all data packages. */
  void deleteAllDataSlot(void);


  /** Closes all data windows. */
  void closeAllSlot();
  

  /** This slot is called when a view window closes.
      @param view a pointer to the closing view. No operations should be carried out on this pointer. */
  void viewIsClosing(ViewCanvas* view);


  /** Displays the "Structure Spacing" dialog. */
  void optionsSlot();


  /** Currently not used. */
  void compareStructuresSlot();


  /** Currently not used. */
  void compareTreesSlot();


  /** Called when an option was changed. */
  void optionChangedSlot();


  /** Turns the "autoexpand reentrancies" option on or off. */
  void toggleAutoExpandSlot(void);


  /** Turns the "windows fit structure size" option on or off. */
  void toggleFitSlot(void);


  /** Turns the "highlight structures" option on or off. */
  void toggleHighlightSlot(void);


  /** Turns the "automatically open windows" option on or off. */
  void toggleAutoOpenSlot(void);


  /** Displays the color selector dialog for the element. */
  void changeNormalColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeTypeColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeTextColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeErrorColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeBackgroundColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeTreeColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeTreeLineColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeLeafColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeArcLabelColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeArcLabelBgColorSlot();

  
  /** Displays the color selector dialog for the element. */
  void changeHiddenColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeMarkerColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeFoundColorSlot();


  /** Displays the color selector dialog for the element. */
  void changeDifferentNodesColorSlot();
  

  /** Displays the font selector dialog for the element. */
  void changeNormalFontSlot();


  /** Displays the font selector dialog for the element. */
  void changeTypeFontSlot();


  /** Displays the font selector dialog for the element. */
  void changeTextFontSlot();


  /** Displays the font selector dialog for the element. */
  void changeErrorFontSlot();


  /** Displays the font selector dialog for the element. */
  void changeTreeFontSlot();


  /** Displays the font selector dialog for the element. */
  void changeLeafFontSlot();


  /** Displays the font selector dialog for the element. */
  void changeArcLabelFontSlot();


  /** Called when the selection in the list of available data packages
      changed. */
  void listSelectionChangedSlot();


  /** Called when the main window closes. */
  void closeEvent(QCloseEvent* ev);


  /** Displays the "About Grisu" dialog. */
  void aboutGrisuSlot(void);

  
  /** Displays the "About KDE" dialog. */
  void aboutKDESlot(void);


  /** Replaces the currently displayed structure or tree with the next data package
      in the list.
      @param replace the ViewCanvas that displays the data to be replaced. */
  void previousStructureSlot(ViewCanvas* replace);


  /** Replaces the currently displayed structure or tree with the previous data package
      in the list.
      @param replace the ViewCanvas that displays the data to be replaced. */
  void nextStructureSlot(ViewCanvas* replace);


  /** Called when the status of the socket connection has changed.
      @param status the new connection status (a <code>GDE_STATUS_xxxx</code> constant,
        see GDataExchange)
      @param hostname the name of the host Grisu is running on
      @param port the number of the port used in the connection. */
  void connectionStateSlot(unsigned char status, char* hostname, int port);


signals:
  /** Emitted when the user wants to change a color. */ 
  void changeColorSignal(int, QWidget*);


  /** Emitted when the user wants to change a font. */ 
  void changeFontSignal(int);


  /** Emitted when the user changed an option. */
  void optionChangedSignal();
};


#endif
