/*  $Id: node.h,v 1.33 2003/05/22 16:43:14 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* Note that flex and bison didn't generate correct C++ code at the time when this
   was initially developed. This has changed in the meantime, however this may
   explain why this code is much more C than C++. */

/**
   @file
   
   This file contains the declarations of the Node class tree and prototypes of C-style manipulation routines for nodes.

   @author Holger Wunsch
*/


#ifndef __NODE_H
#define __NODE_H

#include <stdio.h>

#define NODEFLAG_HIDDEN           0x00000001  /**< Node is hidden */
#define NODEFLAG_EXPANDED         0x00000002  /**< Node is expanded (applies to expandable nodes only) */
#define NODEFLAG_DIFFERENT        0x00000004  /**< Node is to be marked as different (used in structure compare) */
#define NODEFLAG_FOUND            0x00000008  /**< Node is to be marked as found (used in the "Find Node" function) */
#define NODEFLAG_STRIKEOUT        0x00000010  /**< Node is struck out */
#define NODEFLAG_SHOWHIDDEN       0x00000020  /**< Always show node in gray if hidden (even if global "show hidden" is turned off) */
#define NODEFLAG_MULTILINE        0x00000040  /**< Infix operators: draw each operand on a new line */

#define NODETYPE_GENERIC_NODE 1       /**< Type-ID of a GenericNode */
#define NODETYPE_TYPE_NODE 2          /**< Type-ID of a TypeNode */
#define NODETYPE_STRUC_NODE 3         /**< Type-ID of a StrucNode */
#define NODETYPE_FEATVAL_NODE 4       /**< Type-ID of a FeatvalNode */
#define NODETYPE_LIST_NODE 5          /**< Type-ID of a ListNode */
#define NODETYPE_SET_NODE 6           /**< Type-ID of a SetNode */
#define NODETYPE_FUNCTION_NODE 7      /**< Type-ID of a FunctionNode */
#define NODETYPE_REF_NODE 8           /**< Type-ID of a RefNode */
#define NODETYPE_REENT_NODE 9         /**< Type-ID of a ReentNode */
#define NODETYPE_TREE_NODE 10         /**< Type-ID of a TreeNode */
#define NODETYPE_ATOM_NODE 11         /**< Type-ID of a AtomNode */
#define NODETYPE_REL_NODE 12          /**< Type-ID of a RelationNode */
#define NODETYPE_TAIL_NODE 13         /**< Type-ID of a TailNode */
#define NODETYPE_REST_NODE 14         /**< Type-ID of a RestNode */
#define NODETYPE_DISJUNCTION_NODE 15  /**< Type-ID of a DisjunctionNode */
#define NODETYPE_CONJUNCTION_NODE 16  /**< Type-ID of a ConjunctionNode */

/** The maximum length of the string representation of a node ID. */
#define ID_LENGTH 10

class Canvas;
struct DisplayableNode;
class NodeAddress;

/** The GenericNode is the root of the node class hierarchy. Any object that can be 
    displayed by Grisu and that users can interact with is a node.<br><br>
    The GenericNode class provides all properties common to all node types. */
class GenericNode {
public:
  /** The node flags. A combination of <code>NODEFLAG_xxxx</code> values. */
  unsigned long flags;

  /** The nodetype. One of the <code>NODETYPE_xxxx</code> values. */
  unsigned int nodetype;

  /** The node ID as a string. */
  char id[ID_LENGTH];

  /** Sets or clears the NODEFLAG_SHOWHIDDEN flag on a node. If the flag is set,
      and the node is hidden, the node is drawn grayed out even if the global
      "show hidden" option is not set. */
  virtual void hideUnhideChildNodes(Canvas* canvas, NodeAddress& na, bool hide);

  /** Sets the NODEFLAG_SHOWHIDDEN flag on one child node.
      @param canvas the Canvas object that displays the node.
      @param addr the node's address
      @param index the index of the child node. Only nodes count with NODEFLAG_HIDDEN set and
      NODEFLAG_SHOWHIDDEN not set. */
  virtual void unhideChildNode(Canvas* canvas, NodeAddress& addr, int index);
};


/** A list of nodes. */
struct List {
  /** The list item. */
  GenericNode* item;

  /** The next item in the list. */
  List* tail;
};


/** A TypeNode represents a type in afeature structure. */
class TypeNode: public GenericNode {
public:
  /** The name of the type. */
  char* name;
};


/** A FeatvalNode represents one feature (or attribute) in a feature structure, along
    with its value. */
class FeatvalNode: public GenericNode {
public:
  /** The feature's name. */
  char* feature;

  /** The feature's value. */
  GenericNode* value;
};


/** A StrucNode represents an entire feature structure. */
class StrucNode: public GenericNode {
public:
  /** The type of the feature structure. */
  TypeNode* type;

  /** The list of features and their values. */
  List* featvals;

  /** Sets or clears the NODEFLAG_SHOWHIDDEN flag on all nodes in the feature structure. 
      If the flag is set, and the node is hidden, the node is drawn grayed out even if the global
      "show hidden" option is not set. */
  virtual void hideUnhideChildNodes(Canvas* canvas, NodeAddress& na, bool hide);

  /** Sets the NODEFLAG_SHOWHIDDEN flag on one child node.
      @param canvas the Canvas object that displays the node.
      @param addr the node's address
      @param index the index of the child node. Only nodes count with NODEFLAG_HIDDEN set and
      NODEFLAG_SHOWHIDDEN not set. */
  virtual void unhideChildNode(Canvas* canvas, NodeAddress& addr, int index);
};


/** A TailNode represents the tail of a list. Lists are displayed as <a, b | c> (very much like
    the prolog notation). The tail is the structure after the pipe symbol. */
class TailNode: public GenericNode {
public:
  /** The tail. */
  GenericNode* tail;
};


/** A ListNode represents a list of feature structures. Lists are displayed as <a, b | c> (very much like
    the prolog notation). The structure after the pipe symbol is the list's tail. */
class ListNode: public GenericNode {
public:
  /** The list items, a list with elements of GenericNode. */
  List* structs;

  /** The tail. */
  TailNode* tail;
};


/** A RestNode represents all remaining elements in a set, that are not displayed. This works analogously to
    tails in lists. */
class RestNode: public GenericNode {
public:
  /** The rest. */
  GenericNode* rest;
};


/** A SetNode represents a set of feature structures. Sets are displayed as {a, b | c}. The structure after
    the pipe symbol is the rest. */
class SetNode: public GenericNode {
public:
  /** The set elements, a list with elements of GenericNode. */
  List* structs;

  /** The rest. */
  RestNode* rest;
};


/** A FunctionNode represents the head of a function, which takes a number of feature structures as its
    arguments and returns a feature structure. */
class FunctionNode: public GenericNode {
public:
  /** The type. */
  TypeNode* type;

  /** The functor's name. */
  char* functor;

  /** The arguments, a list with elements of type GenericNode. */
  List* structs;
};


/** A RelationNode represents a relation between a number of feature structures. */
class RelationNode: public GenericNode {
public:
  /** The functor's name. */
  char* functor;

  /** The feature stuctures, a list with elements of type GenericNode. */
  List* structs;
};


/** A RefNode represents a reference to another feature structure. RefNodes are identical to tags, the term
    used in HPSG. Tags are used to indicate a point where structure sharing occurs. */
class RefNode: public GenericNode {
public:
  /** The feature structure referenced by this node. This feature structure must be embedded in a
      ReentNode, and ref must be equal to the ReentNode::ref value. RefNodes are displayed by
      a little box that contains a number, which is the same value as ref. */
  int ref;
};


/** A ReentNode is a wrapper around a shared feature structure (or any shared object of another node type).
    ReentNode are referred to by RefNode instances. */
class ReentNode: public GenericNode {
public:
  /** The reference ID. Must be equal to RefNode::ref in a RefNode.<br>This data member should not be confused
      with the unique node ID assigned to every node (see GenericNode). */
  int ref;

  /** The shared node. */
  GenericNode* value;
};


/** A TreeNode represents one visible node in a structure tree. */
class TreeNode: public GenericNode {
public:
  /** The label. */
  char* label;

  /** The label of the arc that leads into this node, or <code>NULL</code> if there is no label (or the node
      is the root node) */
  char* arclabel;

  /** The node ID of the structure to be displayed if the user clicks on the label. */
  char linkid[ID_LENGTH];

  /** The child nodes, a list with elements of type TreeNode. */
  List* subtrees;
};


/** An AtomNode represents an atomic string. Frequently used in PHON lists, for example. */
class AtomNode: public GenericNode {
public:
  /** The string. */
  char* atom;
};


/** A DisjunctionNode represents a disjunction between other nodes. */
class DisjunctionNode: public GenericNode {
 public:
  /** The list of operands. This list must have at least two elements. */
  List* operands;
};


/** A ConjunctionNode represents a conjunction between other nodes. */
class ConjunctionNode: public GenericNode {
 public:
  /** The list of operands. This list must have at least two elements. */
  List* operands;
};


/** NodeIdMapping is one mapping in a <code>NodeIdMap</code>.

    @author Holger Wunsch */
struct NodeIdMapping {
  /** The node-ID. */
  char* id;


  /** Pointer to the node that corresponds to the node-ID. */
  GenericNode* node;


  /** Pointer to the next mapping, or <code>NULL</code>, if this mapping is the last
      element in the map. */
  NodeIdMapping* next;
};  


/** NodeIdMap maps node IDs to pointers to nodes. Used to build a list of all nodes in a
    Document.

    @author Holger Wunsch */
struct NodeIdMap {
  /** Pointer to the first element in a list of mappings. */
  NodeIdMapping* mappings;


  /** The highest node ID contained in the map, represented by an integer number. */
  unsigned int maxID;
};


/** Creates a type node.
    @param flags the node flags
    @param id the node ID
    @param name the type name.
    @return the new TypeNode. */
TypeNode* createType(unsigned int flags, int id, char* name);

/** Creates a StrucNode.
    @param flags the node flags
    @param id the node ID
    @param type the type node containing the type of the feature structure.
    @param featvals a list with instances of FeatvalNode that contains the features and their values.
    @return the StrucNode. */
StrucNode* createStruc(unsigned int flags, int id, TypeNode* type, List* featvals);

/** Creates a FunctionNode.
    @param flags the node flags.
    @param id the node ID.
    @param type the type node containing the return type of the function.
    @param functor a string containing the name of the function
    @param structs the arguments, a list with elements of type GenericNode.
    @return the FunctionNode. */
FunctionNode* createFunction(unsigned int flags, int id, TypeNode* type, char* functor, List* structs);

/** Creates a RelationNode.
    @param flags the node flags.
    @param id the node ID.
    @param functor a string containing the name of the relation
    @param structs the arguments, a list with elements of type GenericNode.
    @return the RelationNode. */
RelationNode* createRelation(unsigned int flags, int id, char* functor, List* structs);

/** Creates a SetNode.
    @param flags the node flags.
    @param id the node ID.
    @param structs the elements in the set (type GenericNode)
    @param rest the rest
    @return the SetNode. */
SetNode* createSet(unsigned int flags, int id, List* structs, RestNode* rest);

/** Creates a ListNode.
    @param flags the node flags.
    @param id the node ID.
    @param structs the items in the list (type GenericNode)
    @param tail the tail
    @return the ListNode. */
ListNode* createList(unsigned int flags, int id, List* structs, TailNode* tail);

/** Creates a FeatvalNode.
    @param flags the node flags.
    @param id the node ID.
    @param feature the feature's name
    @param value the value
    @return the FeatvalNode */
FeatvalNode* createFeatval(unsigned int flags, int id, char* feature, GenericNode* value);

/** Creates a RefNode.
    @param flags the node flags.
    @param id the node ID.
    @param ref the reference ID.
    @return the RefNode. */
RefNode* createRef(unsigned int flags, int id, int ref);

/** Creates a ReentNode.
    @param flags the node flags.
    @param id the node ID.
    @param ref the reference ID.
    @param value the shared node. 
    @return the ReentNode. */
ReentNode* createReent(unsigned int flags, int id, int ref, GenericNode* value);

/** Creates a TreeNode.
    @param flags the node flags.
    @param id the node ID.
    @param label the node label.
    @param arclabel the label on the arc that leads into this node, or <code>NULL</code> if there should be no label.
    @param linkid the node ID of the node to displayed when the tree node is expanded by the user.
    @param subtrees the list of child nodes of type TreeNode.
    @return the TreeNode. */
TreeNode* createTree(unsigned int flags, int id, char* label, char* arclabel, int linkid, List* subtrees);

/** Creates an AtomNode.
    @param flags the node flags.
    @param id the node ID.
    @param atom the value.
    @return the AtomNode. */
AtomNode* createAtom(unsigned int flags, int id, char* atom);

/** Creates a TailNode (which represents the tail in a list of type ListNode).
    @param flags the node flags.
    @param id the node ID.
    @param tail the tail.
    @return the TailNode. */
TailNode* createTail(unsigned int flags, int id, GenericNode* tail);

/** Creates a RestNode (which represents the rest in a set of type SetNode).
    @param flags the node flags.
    @param id the node ID.
    @param rest the rest.
    @return the RestNode. */
RestNode* createRest(unsigned int flags, int id, GenericNode* rest);


/** Creates a DisjunctionNode.
    @param flags the node flags
    @param id the node ID
    @param operands the list of operands. This list must have at least two elements.
    @return the DisjunctionNode. */
DisjunctionNode* createDisjunction(unsigned int flags, int id, List* operands);


/** Creates a ConjunctionNode.
    @param flags the node flags
    @param id the node ID
    @param operands the list of operands. This list must have at least two elements.
    @return the ConjunctionNode. */
ConjunctionNode* createConjunction(unsigned int flags, int id, List* operands);


/** Deallocates all memory used by a node. Note that this function <b>does not</b> recursively
    delete any child nodes! Memory is freed instead by destroying all nodes in a NodeIdMap.
    @param node the node to be destroyed. */
void destroyNode(GenericNode* node);

/** Prepends a node to a list.
    @param item the node
    @param tail the list, or <code>NULL</code> if <code>node</code> is the first element.
    @return a pointer to the list with node as its first element. */
List* prepend(GenericNode* item, List* tail);

/** Appends a node to a list.
    @param l the list or <code>NULL</code> if <code>node</code> is the first element.
    @param item the node
    @return a pointer to the list */
List* append(List* l, GenericNode* item);

/** Frees a list. The nodes in the list are <b>not</b> freed (nodes are freed by deleting
    all nodes in a NodeIdMap.
    @param list the list */
void freeList(List* list);

/** DEBUG: Print a list. */
void catList(List* list, int level);

/** Writes a node and all its child nodes to a file in the Grisu data exchange format.<br>
    (See <a href="http://www.sfs.nphil.uni-tuebingen.de/~wunsch/grisu/doc/ebnf.html">EBNF documentation</a>) */
void writeGenericNode(FILE* file, GenericNode* node);

/** Adds a node to a node map.
    @param map pointer to the node map or <code>NULL</code> if a new node is to be created
    @param node the node
    @param id the node's ID.
    @return pointer to the updated node map. */
NodeIdMap* addToMap(NodeIdMap* map, GenericNode* node, int id);

/** Returns a node given its node ID.
    @param map the node map
    @param id the node ID
    @return a pointer to the node, or <code>NULL</code> if there is no node with <code>id</code> in
    <code>map</code>. */
GenericNode* findNodeById(NodeIdMap* map, char* id);

/** Destroys a node map, and deletes all nodes contained in the map.
    @param map the node map. */
void destroyMap(NodeIdMap* map);

#endif
